// build & run:
//      win:
//          set OMP_NUM_THREADS=10
//          cmake .. && make -j 10 && bin\square.exe
//      win debug:
//          cmake -DCMAKE_BUILD_TYPE=Debug .. && make -j 10 && bin\square.exe
//
//      linux:
//          export OMP_NUM_THREADS=10
//          cmake .. && make -j 10 && ./bin/square
//      linux debug:
//          cmake -DCMAKE_BUILD_TYPE=Debug .. && make -j 10 && ./bin/square

#include "fem.h"
#include "femProblem.h"
#include "femMedium.h"
#include "femDirichlet.h"
#include "femSolver.h"
#include "femPost.h"
#include "OpenMP.h"
#include "build_square.h"
#include "femGmsh.h"
#include <stdexcept>

int main(int argc, char **argv)
{
    try
    {
        // If compiled with OpenMP support (ubuntu), gmsh::initialize
        // also sets the number of threads to "General.NumThreads".
        int nthreads = OpenMP::get_max_threads();
        std::cout << "nthreads = " << nthreads << '\n';
        gmsh::initialize();
        OpenMP::set_num_threads(nthreads);
        gmsh::option::setNumber("General.Terminal", 1);

        // build geometry & mesh
        build_square(0.0, 0.0, 10.0, 1.0, 80, 8, true, true);

        fem::Problem pbl;

        new fem::Medium(pbl, "interior", 100.0, 0.3);

        new fem::Dirichlet(pbl, "left", "X", 0.0);
        new fem::Dirichlet(pbl, "left", "Y", 0.0);
        new fem::Dirichlet(pbl, "pt3", "Y", -0.1);

        fem::Solver solver(pbl);
        solver.solve();

        fem::Post post(solver);
        post.build_views();
        post.show_views( { "Y" });
        post.view();

        // gmsh::fltk::run();

        gmsh::finalize();
        return EXIT_SUCCESS;
    }
    catch (const std::exception &e)
    {
        std::cerr << "\n**ERROR: " << e.what() << '\n';
        return EXIT_FAILURE;
    }
}
